@@ -1,7 +1,7 @@
 /*
  * mainconf.c	Handle the server's configuration.
  *
- * Version:	$Id: mainconfig.c,v 1.36.2.6.2.6 2007/02/09 15:56:11 aland Exp $
+ * Version:	$Id: mainconfig.c,v 1.36.2.6.2.9 2007/04/07 22:06:08 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -399,6 +399,7 @@ static int generate_realms(const char *filename)
 			} else {
 				c->ipaddr = ip_getaddr(authhost);
 				if (c->ipaddr == htonl(INADDR_NONE)) {
+					free(c);
 					radlog(L_ERR, "%s[%d]: Host %s not found",
 					       filename, cf_section_lineno(cs),
 					       authhost);
@@ -410,6 +411,7 @@ static int generate_realms(const char *filename)
 			 * Double check length, just to be sure!
 			 */
 			if (strlen(authhost) >= sizeof(c->server)) {
+				free(c);
 				radlog(L_ERR, "%s[%d]: Server name of length %d is greater than allowed: %d",
 				       filename, cf_section_lineno(cs),
 				       (int) strlen(authhost),
@@ -441,6 +443,7 @@ static int generate_realms(const char *filename)
 			} else {
 				c->acct_ipaddr = ip_getaddr(accthost);
 				if (c->acct_ipaddr == htonl(INADDR_NONE)) {
+					free(c);
 					radlog(L_ERR, "%s[%d]: Host %s not found",
 					       filename, cf_section_lineno(cs),
 					       accthost);
@@ -449,6 +452,7 @@ static int generate_realms(const char *filename)
 			}
 
 			if (strlen(accthost) >= sizeof(c->acct_server)) {
+				free(c);
 				radlog(L_ERR, "%s[%d]: Server name of length %d is greater than allowed: %d",
 				       filename, cf_section_lineno(cs),
 				       (int) strlen(accthost),
@@ -458,6 +462,7 @@ static int generate_realms(const char *filename)
 		}
 
 		if (strlen(name2) >= sizeof(c->realm)) {
+			free(c);
 			radlog(L_ERR, "%s[%d]: Realm name of length %d is greater than allowed %d",
 			       filename, cf_section_lineno(cs),
 			       (int) strlen(name2),
@@ -477,12 +482,14 @@ static int generate_realms(const char *filename)
 		if ((c->ipaddr != htonl(INADDR_NONE)) ||
 		    (c->acct_ipaddr != htonl(INADDR_NONE))) {
 			if ((s = cf_section_value_find(cs, "secret")) == NULL ) {
+				free(c);
 				radlog(L_ERR, "%s[%d]: No shared secret supplied for realm: %s",
 				       filename, cf_section_lineno(cs), name2);
 				return -1;
 			}
 
 			if (strlen(s) >= sizeof(c->secret)) {
+				free(c);
 				radlog(L_ERR, "%s[%d]: Secret of length %u is greater than the allowed maximum of %u.",
 				       filename, cf_section_lineno(cs),
 				       strlen(s), sizeof(c->secret) - 1);
@@ -512,6 +519,7 @@ static int generate_realms(const char *filename)
 
 			c->ldflag = lrad_str2int(ldflags, t, -1);
 			if (c->ldflag == -1) {
+				free(c);
 				radlog(L_ERR, "%s[%d]: Unknown value \"%s\" for ldflag",
 				       filename, cf_section_lineno(cs),
 				       t);
@@ -692,6 +700,7 @@ static RADCLIENT *generate_clients(const char *filename, CONF_SECTION *section)
 				radlog(L_ERR, "%s[%d]: Invalid value '%s' for IP network mask.",
 						filename, cf_section_lineno(cs), netmask + 1);
 				clients_free(list);
+				free(c);
 				return NULL;
 			}
 
@@ -710,6 +719,7 @@ static RADCLIENT *generate_clients(const char *filename, CONF_SECTION *section)
 			radlog(L_CONS|L_ERR, "%s[%d]: Failed to look up hostname %s",
 					filename, cf_section_lineno(cs), hostnm);
 			clients_free(list);
+			free(c);
 			return NULL;
 		}
 
@@ -902,7 +912,10 @@ int proxy_new_listener(void)
 	/*
 	 *	Proxying was not previously defined: die.
 	 */
-	if (last_proxy_port == 0) return -1;
+	if (last_proxy_port == 0) {
+		free(this);
+		return -1;
+	}
 
 	/*
 	 *	Keep going until we find an unused port.
@@ -929,6 +942,7 @@ int proxy_new_listener(void)
 		}
 	}
 
+	free(this);
 	return -1;
 }
 
